cmake_minimum_required(VERSION 3.12)

if(NOT TARGET sirius::sirius)
  # Find bundled modules first  
  set(CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/modules" ${CMAKE_MODULE_PATH})

  # store CXX compiler id. Used in MKL package.
  set(SIRIUS_CXX_COMPILER_ID Clang)
  if(NOT ${CMAKE_CXX_COMPILER_ID})
    set(CMAKE_CXX_COMPILER_ID ${SIRIUS_CXX_COMPILER_ID})
  endif()

  # if not relocatable
  if (NOT NO) 
    set(MPI_ROOT )
    set(ENV{SPG_DIR} )
    set(ENV{LIBXCROOT} ) 
    set(ENV{GSL_ROOT_DIR} ) 
    set(ENV{MKLROOT} )
    set(ENV{MAGMAROOT} )
    set(ENV{ELPAROOT} )
    set(SpFFT_DIR /usr/local/lib/cmake/SpFFT) 
    # TODO: CUDA, ROCM, mpi4py ?
  endif()

  # pass REQUIRED or QUIET depending on top Config call
  if (sirius_FIND_REQUIRED)
    set(mode REQUIRED)
  else()
    set(mode QUIET)
  endif()

  find_package(MPI ${mode} COMPONENTS CXX)
  find_package(GSL ${mode})
  find_package(LibXC 3.0.0 ${mode})
  find_package(LibSPG ${mode})
  find_package(HDF5 ${mode} C)
  include("${CMAKE_CURRENT_LIST_DIR}/hdf5_target.cmake")
  find_package(SpFFT 0.9.6 CONFIG ${mode})
  find_package(SPLA 1.1.0 CONFIG ${mode})
  find_package(costa CONFIG ${mode})

  if(ON)
    find_package(OpenMP ${mode})
  endif()

  if(OFF)
    set(USE_MKL_SHARED_LIBS On) # link against shared MKL libraries
    find_package(MKL ${mode})
  elseif(OFF)
    find_package(CRAY_LIBSCI ${mode})
  else()
    find_package(LAPACK ${mode})
    if(OFF)
      find_package(SCALAPACK ${mode}) # just sets scalapack_DIR
    endif()
  endif()
  
  if(OFF)
    find_package(Elpa ${mode})
  endif()

  if(OFF)
    find_package(MAGMA ${mode})
  endif()

  if(OFF)
    find_package(LibVDWXC 0.3.0 ${mode})
  endif()

  if(OFF)
    # TODO: propagate the passed `sm` flag
    find_package(CUDA ${mode})
    include("${CMAKE_CURRENT_LIST_DIR}/cudalibs_target.cmake")
  endif()

  if(OFF)
    find_package(hip CONFIG ${mode})
    find_package(rocblas CONFIG ${mode})
  endif()

  # Clean-up module path.
  list(REMOVE_ITEM CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/modules")

  # Include SIRIUS target
  include("${CMAKE_CURRENT_LIST_DIR}/siriusTargets.cmake")
endif()
